package net.sf.buildbox.saxutil;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import java.io.File;
import java.io.IOException;

public final class SaxUtils {
    private static final SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
    static {
        saxParserFactory.setNamespaceAware(true);
    }

    private SaxUtils() {
        // not instantiable
    }

    public static <T extends ContentHandler> T parse(File file, T handler) throws IOException, SAXException {
        try {
            createXmlReader(handler).parse(file.getAbsolutePath());
            return handler;
        } catch (ParserConfigurationException e) {
            throw new SAXException(e.getMessage(), e);
        }
    }

    public static <T extends ContentHandler> T parse(InputSource input, T handler) throws IOException, SAXException {
        try {
            createXmlReader(handler).parse(input);
            return handler;
        } catch (ParserConfigurationException e) {
            throw new SAXException(e.getMessage(), e);
        }
    }

    private static XMLReader createXmlReader(ContentHandler handler) throws ParserConfigurationException, SAXException {
        final SAXParser saxParser = saxParserFactory.newSAXParser();
        XMLReader xmlReader = saxParser.getXMLReader();
        xmlReader.setContentHandler(handler);
        return xmlReader;
    }

}
