package net.sf.buildbox.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

public class BbxTimeUtils {
    private static final ThreadLocal<SimpleDateFormat> SVN_FORMAT = new ThreadLocal<SimpleDateFormat>() {
        @Override
        protected SimpleDateFormat initialValue() {
            return new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        }
    };
    public static final ThreadLocal<SimpleDateFormat> ISO_FORMAT = new ThreadLocal<SimpleDateFormat>() {
        @Override
        protected SimpleDateFormat initialValue() {
            return new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        }
    };

    public static Date parseSvnLogTime(String dateStr) throws ParseException {
        final String millisStr = dateStr.substring(0, 23) + "Z";
        return SVN_FORMAT.get().parse(millisStr);
    }

    public static String formatSvnParamTime(Date date) {
        return SVN_FORMAT.get().format(date);
    }

    public static Date utcToLocal(Date localDate) {
        final long localDateMillis = localDate.getTime();
        final int offset = TimeZone.getDefault().getOffset(localDateMillis);
        return new Date(localDateMillis + offset);
    }

    public static Date localToUtc(Date localDate) {
        final long localDateMillis = localDate.getTime();
        final int offset = TimeZone.getDefault().getOffset(localDateMillis);
        return new Date(localDateMillis - offset);
    }

    public static boolean isTimeInRange(Date time, Date startTime, Date endTime) {
        final long timestamp = time.getTime();
        if (startTime != null && timestamp < startTime.getTime()) return false;
        if (endTime != null && timestamp > endTime.getTime()) return false;
        //
        return true;
    }

    public static Date parseIsoTime(String dateStr) throws ParseException {
        final String millisStr = dateStr.substring(0, 23) + "Z";
        return ISO_FORMAT.get().parse(millisStr);
    }

    public static String formatIsoTime(Date date) {
        return ISO_FORMAT.get().format(date);
    }
}
